
/* Paquete  PAQ_CARTERA  para alta, baja y modificacin de datos de las carteras de los clientes.
   Tablas afectadas: CARTERA (id_cartera, NIF,Fecha_alta, Fecha_baja),
                     CARTERA_ACCION (Id_cartera, ticker, cantidad, fecha_inicio, fecha_fin)
   Formado por los procedimientos: IN_CARTERA, DEL_CARTERA, UP_CARTERA

*/



CREATE OR REPLACE PACKAGE PAQ_CARTERA AS 

   PROCEDURE IN_CARTERA ( par_NIF  IN CARTERA.NIF%TYPE,
                          par_Fecha_Alta IN CARTERA.Fecha_Alta%TYPE,
			  par_Fecha_baja IN CARTERA.Fecha_baja%TYPE);

        

  PROCEDURE DEL_CARTERA (par_Id_cartera  IN CARTERA.Id_cartera%TYPE,
                         par_Fecha_baja IN CARTERA.Fecha_baja%TYPE);

        

  PROCEDURE UP_CARTERA (par_Id_cartera  IN CARTERA.Id_cartera%TYPE,
                          par_NIF  IN CARTERA.NIF%TYPE,
                          par_Fecha_Alta IN CARTERA.Fecha_Alta%TYPE,
			  par_Fecha_baja IN CARTERA.Fecha_baja%TYPE);


END PAQ_CARTERA;

/



CREATE OR REPLACE PACKAGE BODY PAQ_CARTERA AS

/*
   NOMBRE:      IN_CARTERA
   DESCRIPCIN: Procedimiento para altas de CARTERAS
                  
   PARMETROS:
       INPUT:  Obligatorios: NIF, Fecha_alta
       OUTPUT:  
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se da de alta la cartera de invesin del cliente
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: 'El NIF no puede ser nulo'
                'ERROR: 'La fecha de alta no puede ser nula'
                'ERROR: (SQLERRM)'
*/

 PROCEDURE IN_CARTERA (par_NIF  IN CARTERA.NIF%TYPE,
                       par_Fecha_Alta IN CARTERA.Fecha_Alta%TYPE,
		       par_Fecha_baja IN CARTERA.Fecha_baja%TYPE) IS
     error EXCEPTION;
     numero       NUMBER;
     mensaje       VARCHAR2(200);
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
     v_usuario	   VARCHAR2(20);


     BEGIN

         SELECT USER into v_usuario
         FROM DUAL;


     /* Hay que comprobar que los parmetros de entrada sean correctos: */

     PAR_IN := par_NIF || ' ' || par_Fecha_Alta || ' ' ||  par_Fecha_baja;     
     
    IF par_NIF IS NULL 
     THEN  RAISE error;
      RSP :='El NIF no puede ser nulo';
     END IF;


    IF par_Fecha_Alta IS NULL 
     THEN  RAISE error;
      RSP :='La fecha de alta no puede ser nula';
     END IF;
    
      -- se inserta en CARTERA los valores de los parmetros
      INSERT INTO CARTERA (Id_cartera,NIF,Fecha_Alta) 
      VALUES (SECUENCIA_CARTERA.NEXTVAL,par_NIF,par_Fecha_Alta);

      -- se almacena en la tabla LOGS lo que se ejecuta para comprobacin.
      INSERT INTO LOG (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado) 
         VALUES(SEQ_Id_log.nextval,SYSDATE,'PK_CARTERA.IN_CARTERA',PAR_IN,'OK',RSP);
       
     
     EXCEPTION

        WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.IN_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.IN_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

   
  END IN_CARTERA;


/*
   NOMBRE: DEL_CARTERA
   DESCRIPCIN: Procedimiento para dar baja carteras de clientes, hay que modificar la tabla e insertar fecha_baja
                  
   PARMETROS:
       INPUT:  Obligatorios: par_Id_cartera, 
       OUTPUT:  res VARCHAR2(300)
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se da de baja la cartera de invesin del cliente
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: 'El NIF no puede ser nulo'
                'ERROR: 'La fecha de baja no puede ser nula'
                'ERROR: (SQLERRM)'
*/

 
 PROCEDURE DEL_CARTERA (par_Id_cartera  IN CARTERA.Id_cartera%TYPE,
                        par_Fecha_baja IN CARTERA.Fecha_baja%TYPE) IS
     error EXCEPTION;
     numero        NUMBER;
     mensaje       VARCHAR2(200);
     v_comprobar   CARTERA.Id_cartera%TYPE;
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
     v_contar	   NUMBER;
     v_usuario     VARCHAR2(20);

     BEGIN

         SELECT USER into v_usuario
         FROM DUAL;


     /* Hay que comprobar que los parmetros de entrada sean correctos: */

     PAR_IN := par_Id_cartera || ' ' || par_Fecha_baja;     
     
   
    IF par_Id_cartera IS NULL 
     THEN  RAISE error;
      RSP :='El identificador de cartera no puede ser nulo';
     END IF;

    IF par_Fecha_baja IS NULL 
     THEN  RAISE error;
      RSP :='La fecha de baja no puede ser nula';
     END IF;


   -- Hay que comprobar que el identificador de cartera exista.
    BEGIN
         SELECT fecha_baja INTO v_comprobar
         FROM CARTERA
         WHERE id_cartera = par_Id_cartera;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El identificador de cartera no existe';
           RAISE error;
   END;


  -- Hay que comprobar que el identificador de cartera no tenga acciones de alta.
   
   BEGIN
         SELECT count(ticker) INTO v_contar
         FROM CARTERA_ACCION
         WHERE id_cartera = par_Id_cartera AND fecha_fin is null;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El identificador de cartera no existe';
           RAISE error;
   END;
        -- se realizan las comprobaciones 
     
           IF v_contar =0 then 
      
              UPDATE CARTERA SET Fecha_baja = par_Fecha_baja
              WHERE Id_cartera = par_Id_cartera;


               -- para comprobar lo que se ejecuta.
                 INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
                 VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'OK',v_usuario);
       
           ELSE
 
              RSP:= par_Id_cartera || ' tiene acciones activas';
              RAISE error;

           END IF;

     EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

   
  END DEL_CARTERA;


  
/* Procedimiento para modificar en CARTERA, hay que modificar la tabla CARTERA 
    e insertar los valores de los parmetros */
  
   PARAMETROS:
        INPUT: par_Id_cartera  IN CARTERA.Id_cartera%TYPE,
               par_Fecha_Alta IN CARTERA.Fecha_Alta%TYPE,
               par_Fecha_baja IN CARTERA.Fecha_baja%TYPE

        OUTPUT:  RSP 
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se modifican los datos de la cartera del cliente.
   EXCEPTION:   'ERROR: El id_cartera del cliente no puede ser nulo
                
                'ERROR: (SQLERRM)'
*/

  PROCEDURE UP_CARTERA (par_Id_cartera  IN CARTERA.Id_cartera%TYPE,
                          par_Fecha_Alta IN CARTERA.Fecha_Alta%TYPE,
			  par_Fecha_baja IN CARTERA.Fecha_baja%TYPE);
    IS

   BEGIN 

   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_NIF         CARTERA.NIF%TYPE;
   v_id_cartera  CARTERA.id_cartera%TYPE;
   v_fecha_baja  CARTERA.id_fecha_baja%TYPE;
   error 	 EXCEPTION;
   v_usuario     VARCHAR2(30);

         SELECT USER into v_usuario
         FROM DUAL;

     PAR_IN :=  par_Id_cartera || ' ' || par_Fecha_alta || ' ' || par_Fecha_baja;


  -- Hay que comprobar que los parmetros de entrada sean correctos: 
   
     -- se comprueba que el par_Id_cartera no sea nulo.
     RSP := 'ERROR: El Id_cartera no puede ser nulo';
     IF par_Id_cartera IS NULL THEN  RAISE error; END IF;

     -- se comprueba que el par_Fecha_Alta no sea nulo.
     RSP := 'ERROR: La fecha de alta no puede ser nula';
     IF par_Id_cartera IS NULL THEN  RAISE error; END IF;


        
         -- Hay que comprobar que el Id_cartera exista en la base de datos.
         BEGIN
 	         SELECT Id_cartera INTO v_id_cartera
                 FROM CARTERA
                 WHERE Id_cartera = par_Id_cartera;
         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: El id_cartera no existe en la base de datos';
                  RAISE error;
         END;

        -- Hay que tener cuidado porque no es conveniente modificar la fecha de la baja de carteras con acciones activas.
        -- la fecha de baja puede ser distinta a null

        BEGIN
                  SELECT count( ticker) INTO v_contar
                   FROM CARTERA_ACCION
                  WHERE id_cartera = par_Id_cartera AND fecha_fin is null;
           EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    RSP := 'ERROR: El id_cartera de cartera no existe en la base de datos';
                    RAISE error;
         END;
        
               -- si v_contar es igual a cero entonces las acciones no estn activas.
              IF v_contar =0 then
 
               -- actualizo la tabla CARTERA con los datos de los parmetros.

              UPDATE CARTERA SET Fecha_baja = par_Fecha_baja, Fecha_alta=par_Fecha_Alta
              WHERE Id_cartera = par_Id_cartera;


               -- para comprobar lo que se ejecuta.
               INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
                 VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.UP_CARTERA',PAR_IN,RSP,'OK',v_usuario);
              END IF;

   EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.UP_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.UP_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);
   END;

  
END PAQ_CARTERA;

